;-----
;Create Menulib library
;-----
CreateLib "MenuLib"
;-----
;Main procedure...
;-----
Proc CenterMenu(SCREEN_PROC)
;-----
;Variable declaration
;-----
Private  CHR_VAR1,    ;Relates Row-Position to ASCII Values (Caps)
         CHR_VAR2,    ;Relates Row-Position to ASCII Values (Lowercase)
         SCREEN_PROC, ;Menu Text - See description below
         T,           ;Top row choice
         B,           ;Bottom row choice
         L,           ;Left edge of hilite bar
         R            ;Right edge of hilite bar
         ;HILITE      Hilight bar color code
         ;LOLITE      Color code for entire screen
;-----
;Start clean
;-----
Reset
;-----
;Canvas off BEFORE it's cleared...stays off till completely painted.
;-----
Canvas OFF
;-----
;Clear canvas AFTER it has been turned off.
;-----
Clear
;-----
;Turn cursor off - remember to turn it on if necessary..
;-----
Cursor Off
;-----
;Declares menulib the auto-search lib
;-----
AutoLib="MenuLib"
;-----
;Positions pal cursor at top-left of screen
;-----
@0,0
;-----
;Calls SCREEN_PROC procedure - this will display the actual
;text of the menu.
;-----
ExecProc SCREEN_PROC
;-----
;Release SCREEN_PROC from memory
;-----
Release Procs SCREEN_PROC
;-----
;Assigns value to Ascii keys based on value of 'T'variable
;-----
CHR_VAR1=65-T
CHR_VAR2=CHR_VAR1+32
;-----
;This line is optional...will return to last menu position
;-----
If Not IsAssigned(X) Then X=T EndIf
;-----
;Paint entire canvas LOLITE and highlight bar HILITE
;-----
PaintCanvas Attribute LOLITE 0,0,24,79
PaintCanvas Attribute HILITE X,L,X,R
;-----
;Set style attribute to match screen (for date/time)
;-----
Style Attribute LOLITE
;-----
;Screen is drawn, make it visible to user by turning canvas on
;-----
Canvas On
;-----
;This 'while' loop will flow until user selects menu item by cursoring
;and hitting <Enter> or by choosing the associated menu letter - or, of
;course, it he/she hits <Esc> to quit...
;-----
While True
;-----
;This 'while' loop flows until a key is pressed - keeps the time/date
;continually updated
;-----
 While Not CharWaiting()
   @23,2??Format("D2",Today())
   @23,77-Len(DaveTime())??DaveTime()
 EndWhile
 ;-----
 ;Accepts user's keypress and assigns it to the variable 'Choice'
 ;-----
 Choice=GetChar()
   Switch
      ;-----
      ;F5 - F8 could be considered 'Hot Keys' - assigned to display a
      ;quick graph, or telephone list, etc...
      ;FMess is a procedure I use in place of PAL's 'Message' command
      ; F5
      ;-----
      Case Choice=-63   :  FMess("1000`F5")
      ;-----
      ; F6
      Case Choice=-64   :  FMess("1500`Drive C: "+StrVal(Format("W13,EC",Drivespace("C")))+" Free")
      ;-----
      ; F7
      Case Choice=-65   :  FMess("1500`Memory: "+StrVal(Format("W12,EC",RMemLeft()))+" Available")
      ;-----
      ; F8
      Case Choice=-66   :  FMess("2000`Written by David Reed - 102254,1251")
      ;-----
      ;If user presses <Down> or <Up>, X value is incremented accordingly,
      ;unless it surpasses the SCREEN_PROC defined limits
      ; DOWN
      ;-----
      Case Choice=-80   :  PaintCanvas Attribute LOLITE X,L,X,R
                           If X > B-1 Then
                              X=T
                              Else X=X+1
                           EndIf
                           PaintCanvas Attribute HILITE X,L,X,R
      ;-----
      ; UP
      Case Choice=-72   :  PaintCanvas Attribute LOLITE X,L,X,R
                           If X < T+1 Then
                              X=B
                              Else X=X-1
                           EndIf
                           PaintCanvas Attribute HILITE X,L,X,R
      ;-----
      ;If the user presses <Enter>, the variable THIS_KEY is assigned
      ;the relative Ascii value character - and XX is initialized and
      ;given a value equal to X - You'll see why when you get a few
      ;lines down...
      ; ENTER
      ;-----
      Case Choice=13    :  THIS_KEY=Chr(X+CHR_VAR1)
                           XX=X
                           Sleep 100
                           QuitLoop
      ;-----
      ; ESC
      Case Choice=27    :  FMess("1000`Exiting to Paradox")
                           Quit
      ;-----
      ;If the user presses one of the valid menu-choice keys, such as
      ;"A" or "B", THIS_KEY is assigned that keypress value, and XX is
      ;initialized.
      ; VALID CAPS
      ;-----
      Case Choice > 64  And Choice <= B+CHR_VAR1  :
                           THIS_KEY=Chr(Choice)
                           XX=Choice-CHR_VAR1
                           QuitLoop
      ;-----
      ; VALID LOWERCASE
      Case Choice > 96 And Choice <= B+CHR_VAR2   :
                           THIS_KEY=Upper(Chr(Choice))
                           XX=Choice-(CHR_VAR2)
                           QuitLoop
      ;-----
      ;An invalid keypress beeps twice and loops to top
      ; INVALID KEYPRESS
      ;-----
      Otherwise   :        Beep Beep
                           FMess("Not a valid choice")
   EndSwitch
EndWhile

;-----
;If the menu selection chosen is not the same as the menu selection
;which the hilight bar is resting on (through a user keypress of the
;selection letter such as "A" or "B"), the user's choice will be
;highlighted for 3/10 second so as to visually endorse the keypress.
;-----
If XX <> X Then
   PaintCanvas Attribute LOLITE X,L,X,R
   PaintCanvas Attribute HILITE XX,L,XX,R
   Sleep 300
   X=XX
EndIf
;-----
;THIS_KEY has been assigned a string value ("A","B", etc) based on the
;user's choice - this value is returned to the parent procedure for
;action based on THIS_KEY's value.
;-----
Return THIS_KEY

EndProc
WriteLib "MenuLib" CenterMenu
Release Procs CenterMenu




;-----
;FMess Procedure Replaces the Pdox 'Message' command
;-----
Proc FMess(F_STR)
   If SubStr(Type(F_STR),1,1) <> "A" Then
      F_STR=StrVal(F_STR)
   EndIf

   If Len(F_STR) > 60 Then
      F_STR=SubStr(F_STR,1,60)
   EndIf

   Style Attribute LOLITE
   @22,Int((80-Len(F_STR))/2)??F_STR
   Sleep 500

   @22,Int((80-Len(F_STR))/2)??Fill(" ",Len(F_STR))
EndProc
WriteLib "MenuLib" FMess
Release Procs Fmess


;-----
;This is where menu text is defined. Other than keeping text shorter than
;22 lines, and making sure all menu choices fall between the column values
;'L' and 'R' (for Left and Right), the text can consist of anything:  Main
;Menu, Help screen, Options list, etc
;-----
Proc Menu_1_Text()
T=6              ;Row number of top-most menu item
B=12             ;Row number of last menu item
L=27             ;Column number for left edge of the highlight bar
R=52             ;Column number for right edge of the highlight bar
;---
;In this demo, keep text shorter than 22 lines-time & date display on
;line 22...
Text

                       ͻ
                    ͻ
                             Demonstration Menu          
                    ͼ
                     ͻ
                             A - Procedure A            
                             B - Default Screen         
                             C - Mono Screen            
                             D - Pink Screen            
                             E - Cyan Screen            
                             F - Procedure F            
                            Esc - Exit to Pdox          
                     ͼ
                    ͻ
                         F5 -           F7 - FreeMem     
                         F6 - FreeDisk  F8 - Message     
                    ͼ
                       ͼ

EndText
EndProc
WriteLib "MenuLib" Menu_1_Text
Release Procs Menu_1_Text


;-----
;Highest level procedure looks like this.  *Menus can be nested...
;Keep in mind that RetVal can be assigned the first letter of menu
;choices, if that's easier for your particular client.
;-----
Proc StartUp()
Release Vars All
Clear
Reset
AutoLib="MenuLib"
DefaultScreen()
While True
   CenterMenu("Menu_1_Text")
   Release Procs CenterMenu
    Switch
      Case RetVal="A" : FMess("800`Procedure A")
      Case RetVal="B" : DefaultScreen()
      Case RetVal="C" : MonoScreen()
      Case RetVal="D" : PinkScreen()
      Case RetVal="E" : CyanScreen()
      Case RetVal="F" : FMess("800`Procedure F")
      Case RetVal="G" : FMess("800`Quitting") Quit
      Case RetVal=False : FMess("800`Quitting") Quit
      Otherwise : FMess("Invalid choice")
    EndSwitch
EndWhile
EndProc
WriteLib "MenuLib" StartUp
Release Procs StartUp


;-----
;FMess Procedure (Flash-Message) Replaces the Pdox 'Message' command
;-----
;Syntax - FMess("[SleepLength][`]DisplayString")
;'SleepLength' and ` are optional - default=500 (1/2 sec)
;IMPORTANT: Do not include ` (Ascii 96) in the string unless you are
;specifying a different 'SleepLength' variable...
;-----
Proc FMess(F_STR)
Private F_STR

  If SubStr(Type(F_STR),1,1) <> "A" Then  ;Makes sure it's a string
     F_STR=StrVal(F_STR)
  EndIf
  If Len(F_STR) > 60 Then
     F_STR=SubStr(F_STR,1,60)
  EndIf
  If Search("`",F_STR) <= 1 Then
     @22,Int((80-Len(F_STR))/2)??F_STR
     Sleep 500
  Else
     SLP_VAR=NumVal(SubStr(F_STR,1,Search("`",F_STR)-1))
     F_STR=SubStr(F_STR,Search("`",F_STR)+1,Len(F_Str)-Search("`",F_STR))
     @22,Int((80-Len(F_STR))/2)??F_STR
     Sleep SLP_VAR
  EndIf
@22,Int((80-Len(F_STR))/2)??Fill(" ",Len(F_STR))
EndProc
WriteLib "MenuLib" FMess
Release Procs Fmess


;-----
;DaveTime formats the time the way I like it...(Using the NumVal to
;StrVal filter drops the leading zero, if necessary)
;-----
Proc DaveTime()
   If Time() > "12:59:59" Then
      HourVar=StrVal(NumVal(SubStr(Time(),1,2))-12)
   Else
      HourVar=StrVal(NumVal(SubStr(Time(),1,2)))
   EndIf
If HourVar="0" Then HourVar="12" EndIf
   If Time() > "11:59:59" Then
      Return HourVar+SubStr(Time(),3,3)+" PM"
   Else
      Return HourVar+SubStr(Time(),3,3)+" AM"
   EndIf
EndProc
WriteLib "MenuLib" DaveTime
Release Procs DaveTime



;-----
;Different screen colors, easily assigned
;-----
Proc MonoScreen()
LOLITE=10
HILITE=47
EndProc
WriteLib "MenuLib" MonoScreen
Release Procs MonoScreen


Proc PinkScreen()
LOLITE=77
HILITE=12
EndProc
WriteLib "MenuLib" PinkScreen
Release Procs PinkScreen

Proc CyanScreen()
LOLITE=48
HILITE=31
EndProc
WriteLib "MenuLib" CyanScreen
Release Procs CyanScreen


Proc DefaultScreen()
LOLITE=31
HILITE=63
EndProc
WriteLib "MenuLib" DefaultScreen
Release Procs DefaultScreen

